using Gtk;
using Granite;


namespace Eidete.Widgets{
	
	public class SelectionArea : Granite.Widgets.CompositedWindow{
	
		private int[,] pos;
		private Label size;
		public bool discrete;
		public Box container;
		
		public int x;
		public int y;
		public int w;
		public int h;
		
		public SelectionArea (){
			this.stick ();
			this.resizable = true;
			this.set_has_resize_grip (false);
			this.set_default_geometry (640, 480);
			this.events = Gdk.EventMask.BUTTON_MOTION_MASK | Gdk.EventMask.BUTTON1_MOTION_MASK | 
				Gdk.EventMask.BUTTON_PRESS_MASK | Gdk.EventMask.BUTTON_RELEASE_MASK ;
			
			this.button_press_event.connect ( (e) => {
				Gdk.WindowEdge [] dir = {Gdk.WindowEdge.NORTH_WEST,
					Gdk.WindowEdge.NORTH,Gdk.WindowEdge.NORTH_EAST,
					Gdk.WindowEdge.EAST,Gdk.WindowEdge.SOUTH_EAST,Gdk.WindowEdge.SOUTH,
					Gdk.WindowEdge.SOUTH_WEST,Gdk.WindowEdge.WEST};
				for (var i=0;i<8;i++){
					if (in_quad (pos[i,0]-10, pos[i,1]-10, 20, 20, (int)e.x, (int)e.y)){
						this.begin_resize_drag (dir[i], (int)e.button, (int)e.x_root, (int)e.y_root, e.time);
						return false;
					}
				}
				this.begin_move_drag ((int)e.button, (int)e.x_root, (int)e.y_root, e.time);
				return false;
			});
		
			this.configure_event.connect ((e) => {
				size.label = "<span size='20000' color='#fbfbfb'>"+e.width.to_string ()+"x"+
					e.height.to_string ()+"</span>";
				x = e.x;
				y = e.y;
				w = e.width;
				h = e.height;
				geometry_changed (e.x, e.y, e.width, e.height);
				return false;
			});
		
			size = new Label ("<span size='20000' color='#fbfbfb'>640x480</span>");
			size.use_markup = true;
		
			container = new Box (Orientation.HORIZONTAL, 0);
			container.pack_start (size);
			container.expand = false;
			container.halign = container.valign = Align.CENTER;
			container.draw.connect ( (ctx) => {
				Granite.Drawing.Utilities.cairo_rounded_rectangle (ctx, 0, 0, 
					size.get_allocated_width (), size.get_allocated_height (), 8);
				ctx.set_source_rgba (0.1, 0.1, 0.1, 0.8);
				ctx.fill ();
				return false;
			});
		
			this.add (container);
		}
	
		private bool in_quad (int qx, int qy, int qh, int qw, int x, int y){
			return ((x>qx) && (x<(qx+qw)) && (y>qy) && (y<qy+qh));
		}
	
		public override bool draw (Cairo.Context ctx){
			int w = this.get_allocated_width  ();
			int h = this.get_allocated_height ();
			if (!discrete){
				pos = {{10,10},{w/2,10},{w-10,10},{w-10,h/2},{w-10,h-10},{w/2,h-10},{10,h-10},{10,h/2}};
				
				ctx.rectangle (10, 10, w-20, h-20);
				ctx.set_source_rgb (0.6, 0.6, 0.6);
				ctx.set_line_width (5.0);
				ctx.stroke ();
				ctx.rectangle (11, 10, w-18, h-18);
				ctx.set_source_rgba (0.1, 0.1, 0.1, 0.3);
				ctx.fill ();
		
				for (var i=0;i<8;i++){
					ctx.arc (pos[i,0], pos[i,1], 10, 0.0, 2 * 3.14);
					ctx.set_source_rgb (0.898, 0.898, 0.898);
					ctx.fill ();
					ctx.arc (pos[i,0], pos[i,1], 11, 0.0, 2 * 3.14);
					ctx.set_line_width (1.0);
					ctx.set_source_rgba (0, 0, 0, 0.3);
					ctx.stroke ();
				}
			}else{
				ctx.rectangle (0, 0, w, h);
				ctx.set_source_rgb (0.8, 0.0, 0.0);
				ctx.set_line_width (3.0);
				ctx.stroke ();
			}
			return base.draw (ctx);
		}
		
		public signal void geometry_changed (int x, int y, int width, int height);
		
		public void to_discrete (){
			this.discrete = true;
			this.set_keep_above (true);
			this.queue_draw ();
			this.remove (container);
			this.resize (w+5, h+5);
			this.move (x-1, y-1);
			
			this.enter_notify_event.connect ( () => {
				Cairo.RectangleInt rect = {0, 0, 1, 1};
				this.get_window ().input_shape_combine_region (new Cairo.Region.rectangle (rect), 0, 0);
				return true;
			});
		
		}
	}
}


