//valac --pkg gtk+-3.0 --pkg gstreamer-0.10 --pkg gstreamer-pbutils-0.10 --pkg granite screencast.vala

using Gst;
using Granite;
using Gtk;

public void* test (){
	return null;
}

namespace Eidete{
	
	
	class LLabel : Label{
		public LLabel (string label){
			this.set_halign (Align.START);
			this.label = label;
		}
		public LLabel.indent (string label){
			this (label);
			this.margin_left = 10;
		}
		public LLabel.markup (string label){
			this (label);
			this.use_markup = true;
		}
		public LLabel.right (string label){
			this.set_halign (Align.END);
			this.label = label;
		}
		public LLabel.right_with_markup (string label){
			this.set_halign (Align.END);
			this.use_markup = true;
			this.label = label;
		}
	}
	
	
	public struct Settings{
		public int sx;
		public int sy;
		public int ex;
		public int ey;
		public bool audio;
		public bool keyview;
		public string format;
		public string destination;
		public string recorder;
	}
	
	public class EideteApp : Granite.Application{
		
		construct{
			program_name = "Eidete";
			exec_name = "eidete";
			
			build_data_dir = Constants.DATADIR;
			build_pkg_data_dir = Constants.PKGDATADIR;
			build_release_name = Constants.RELEASE_NAME;
			build_version = Constants.VERSION;
			build_version_info = Constants.VERSION_INFO;
			
			app_years = "2011";
			app_icon = "";
			app_launcher = "eidete.desktop";
			application_id = "net.launchpad.Eidete";
			
			main_url = "https://code.launchpad.net/eidete";
			bug_url = "https://bugs.launchpad.net/eidete";
			help_url = "https://code.launchpad.net/eidete";
			translate_url = "https://translations.launchpad.net/eidete";
			
			about_authors = {"Tom Beckmann <tombeckmann@online.de>"};
			about_documenters = {"Tom Beckmann <tombeckmann@online.de>"};
			about_artists = {""};
			about_comments = "Development release, not all features implemented";
			about_translators = "";
			about_license_type = License.GPL_3_0;
		}
		
		public dynamic Pipeline pipeline;
		
		public Window main_window;
		public Eidete.Widgets.KeyView keyview;
		public Eidete.Widgets.SelectionArea selectionarea;
		public Grid grid;
		public Button start_bt;
		public Button stop_bt;
		
		public Settings settings;
		
		private bool recording;
		private bool started_recording;
		private bool typing_size;
		
		public EideteApp (){
			this.main_window = new Window ();
			this.main_window.title = "Eidete";
			this.started_recording = false;
			
			/*
			  UI
			*/
			
			this.grid = new Grid ();
			grid.margin = 12;
			grid.column_spacing = 12;
			grid.row_spacing = 6;
			
			start_bt = new Button ();
			start_bt.image = new Image.from_stock (Stock.MEDIA_RECORD, IconSize.DIALOG);
			start_bt.relief = ReliefStyle.NONE;
			start_bt.set_tooltip_text ("Start/Continue recording");
			
			stop_bt = new Button ();
			stop_bt.image = new Image.from_stock (Stock.MEDIA_STOP, IconSize.DND);
			stop_bt.relief = ReliefStyle.NONE;
			stop_bt.set_sensitive (false);
			stop_bt.set_halign (Align.START);
			stop_bt.set_tooltip_text ("Finish recording");
			
			var size = new Box (Orientation.HORIZONTAL, 5);
			var width = new Entry ();
			width.text = "1920";
			width.width_chars = 5;
			size.pack_start (width);
			size.pack_start (new Label ("x"));
			var height = new Entry ();
			height.text = "1080";
			height.width_chars = 5;
			size.pack_start (height);
			size.set_sensitive (false);
			
			var use_audio      = new Switch ();
			var use_keyview    = new Switch ();
			var use_arealimit = new Switch ();
			
			var filename = new Button ();
			filename.image = new Image.from_stock (Stock.SAVE, IconSize.BUTTON);
			filename.label = "Choose...";
			
			grid.attach (start_bt, 0, 0, 1, 2);
			grid.attach (stop_bt,  1, 1, 1, 1);
			grid.attach (new LLabel.right (" "), 0, 2, 2, 1); //pro way for creating an empty line
			grid.attach (new LLabel.right ("Audio recording"), 0, 4, 1, 1);
			grid.attach (use_audio, 1, 4, 1, 1);
			grid.attach (new LLabel.right ("Keypresses"), 0, 5, 1, 1);
			grid.attach (use_keyview, 1, 5, 1, 1);
			grid.attach (new LLabel.right ("Area limit"), 0, 6, 1, 1);
			grid.attach (use_arealimit, 1, 6, 1, 1);
			grid.attach (size, 0, 7, 2, 1);
			
			
			/*
			  Events
			*/
			
			this.start_bt.clicked.connect ( () => {
				if (!this.started_recording){
					stop_bt.set_sensitive (true);
					var count = new Eidete.Widgets.Countdown ();
					count.start (this); (settings.destination);
					this.main_window.iconify ();
					this.started_recording = true;
				}else{
					this.main_window.iconify ();
					this.recording = true;
					this.pipeline.set_state (State.PLAYING);
				}
			});
			
			this.stop_bt.clicked.connect ( () => {
				var end = new Eidete.Widgets.EndDialog (this);
				end.show_all ();
				this.main_window.hide ();
			});
			
			settings.sx = 0;
			settings.sy = 0;
			settings.ex = 0;
			settings.ey = 0;
			use_arealimit.active = false;
			use_arealimit.button_press_event.connect ( () => { //TODO make this reliable
				if (!use_arealimit.active){
					use_arealimit.active = true;
					selectionarea = new Eidete.Widgets.SelectionArea ();
					selectionarea.show_all ();
					size.set_sensitive (true);
					selectionarea.geometry_changed.connect ( (x, y, w, h) => {
						if (!typing_size){
							width.text  = w.to_string ();
							height.text = h.to_string ();
							settings.sx = x;
							settings.sy = y;
							settings.ex = settings.sx+w;
							settings.ey = settings.sy+h;
						}
					});
				}else{
					use_arealimit.active = false;
					selectionarea.destroy ();
					size.set_sensitive (false);
				}
				return false;
			});
			
			width.key_release_event.connect ( (e) => {
				selectionarea.resize (int.parse (width.text), int.parse (height.text));
				typing_size = true;
				return false;
			});
			height.focus_out_event.connect ( (e) => {typing_size = false;return false;});
			height.key_release_event.connect ( (e) => {
				selectionarea.resize (int.parse (width.text), int.parse (height.text));
				typing_size = true;
				return false;
			});
			height.focus_out_event.connect ( (e) => {typing_size = false;return false;});
			
			settings.audio = false;
			use_audio.active = false;
			use_audio.button_press_event.connect ( () => {
				settings.audio = !use_audio.active;
				return true;
			});
			
			settings.keyview = false;
			use_keyview.active = false;
			use_keyview.button_press_event.connect ( () => {
				settings.keyview = !use_keyview.active;
				return true;
			});
			
			settings.destination = GLib.Environment.get_tmp_dir ()+
			    "/screencast"+new GLib.DateTime.now_local ().to_unix ().to_string ()+".webm";
			
			
			this.main_window.add (grid);
			
			
			
			
			this.main_window.focus_in_event.connect ( (event) => {
				if (recording){
					pipeline.set_state (State.PAUSED);
					this.recording = false;
				}
				return false;
			});
			
			this.main_window.destroy.connect ( () => {
				if (pipeline != null)
					pipeline.set_state (State.NULL);
				Gtk.main_quit ();
			});
		}
		
		public void record (){
			this.recording = true;
			
			if (settings.keyview){
				keyview = new Eidete.Widgets.KeyView ();
				if (selectionarea != null)
					keyview.place (selectionarea.x+selectionarea.w-100,
					               selectionarea.y+selectionarea.h-100);
				else
					keyview.place (Gdk.Screen.get_default ().get_width  () - 100, 
					               Gdk.Screen.get_default ().get_height () - 100);
				keyview.show_all ();
			}
			
			pipeline                   = new Pipeline ("screencast-pipe");
			dynamic Element muxer      = ElementFactory.make ("webmmux", "mux");
			dynamic Element sink       = ElementFactory.make ("filesink", "sink");
			
			//video bin
			Gst.Bin videobin = new Gst.Bin ("video");
			try {
				videobin = (Gst.Bin)parse_bin_from_description (
				    "ximagesrc name=\"videosrc\" ! video/x-raw-rgb, framerate=15/1 ! 
				     ffmpegcolorspace ! vp8enc name=\"encoder\" ! queue", true);
			}catch (Error e){ stderr.printf ("Error: %s\n", e.message);}
			
			//audio bin
			Gst.Bin audiobin = new Gst.Bin ("audio");
			try{
				audiobin = (Gst.Bin)parse_bin_from_description ("autoaudiosrc name=\"audiosrc\" ! 
				                  audioconvert ! audioresample ! audiorate ! vorbisenc ! queue", true);
			}catch (Error e){ stderr.printf ("Error: %s\n", e.message);}
			
			pipeline.set_state (State.READY);
			
			//configure
			sink.set ("location", settings.destination);
			videobin.get_by_name ("videosrc").set ("startx", this.settings.sx);
			videobin.get_by_name ("videosrc").set ("starty", this.settings.sy);
			videobin.get_by_name ("videosrc").set ("endx",   this.settings.ex-1);
			videobin.get_by_name ("videosrc").set ("endy",   this.settings.ey-1);
			videobin.get_by_name ("videosrc").set ("use-damage", false);
			
			videobin.get_by_name ("encoder").set  ("quality", 10.0);
			videobin.get_by_name ("encoder").set  ("speed", 2);
			videobin.get_by_name ("encoder").set  ("threads", 2);
			
			
			if (pipeline==null||muxer==null||sink==null||videobin==null||audiobin==null){
				stderr.printf ("Error: Elements weren't made correctly!\n");
			}
			
			if (settings.audio)
				pipeline.add_many (audiobin, videobin, muxer, sink);
			else
				pipeline.add_many (videobin, muxer, sink);
			
			
			videobin.get_static_pad ("src").link(muxer.get_request_pad ("video_%d"));
			if (settings.audio)
				audiobin.get_static_pad ("src").link(muxer.get_request_pad ("audio_%d"));
			
			muxer.link (sink);
			
			pipeline.get_bus ().add_watch (bus_message_cb);
			pipeline.set_state (State.PLAYING);
			
			
			if (selectionarea != null)
				selectionarea.to_discrete ();
			
		}
		
		private bool bus_message_cb (Gst.Bus bus, Message msg){
			switch (msg.type){
				case Gst.MessageType.ERROR:
					GLib.Error err; string debug;
					msg.parse_error (out err, out debug);
					print ("Error: %s\n", err.message);
					pipeline.set_state (State.NULL);
					break;
				case Gst.MessageType.EOS:
					print ("Finished, why??\n");
					pipeline.set_state (State.NULL);
					break;
				case Gst.MessageType.STATE_CHANGED:
					Gst.State oldstate; Gst.State newstate; Gst.State pending;
					msg.parse_state_changed (out oldstate, out newstate, out pending);
					print ("state changed: %s->%s:%s\n", oldstate.to_string (), newstate.to_string (), pending.to_string ());
					break;
				default:
					break;
			}
			return true;
		}
	}
	
}


public static void main (string [] args){
	Gst.init (ref args);
	Gtk.init (ref args);
	
	var eidete = new Eidete.EideteApp ();
	eidete.main_window.show_all ();
	
	Gtk.main ();
}

